/********************************************************************************
 * xxxx.c
 *
 * DESCRIPTION:
 *       MS API initialization routine
 *
 * DEPENDENCIES:   Platform
 *
 * FILE REVISION NUMBER:
 *
 *******************************************************************************/
#define DEBUG
#include "msSample.h"
#include <errno.h>
#include <stdio.h>
#include <ctype.h>
#include <fcntl.h>
#include <signal.h>
#include <gtHwCntl.h>

#define PROPERTY_VLAN_TYPE	"vlan"
#define PROPERTY_VLAN_LAN_VID	"lanVid"
#define PROPERTY_VLAN_LAN_PRIO	"lanPri"
#define PROPERTY_VLAN_PC_VID	"pcVid"
#define PROPERTY_VLAN_PC_PRIO	"pcPri"
#define MAXLEN 			80

GT_SYS_CONFIG   cfg;
GT_QD_DEV       diagDev;
GT_QD_DEV       *dev=&diagDev;

static int fd;
static struct ifreq ifr;


GT_STATUS MiiInit(GT_QD_DEV* dev)
{
	struct mii_ioctl_data *mii;

	memset(&ifr, 0, sizeof(ifr));
	strcpy(ifr.ifr_ifrn.ifrn_name, "eth0");
	mii = (struct mii_ioctl_data *)(&ifr.ifr_ifru.ifru_data);
	mii->phy_id = 0x115;  /* phy's ID */

	mii->reg_num = 3;
	mii->val_in = 0;
	mii->val_out = 0;

	fd = socket(AF_INET, SOCK_DGRAM, 0);

	if (fd < 0) {

		printf("DEBUG:Can't create socket for MII \n");
		return GT_FAIL;
	}

	return GT_OK;
}


GT_BOOL MiiRead (GT_QD_DEV* dev, unsigned int portNumber , unsigned int MIIReg,
		unsigned int* value)
{
	struct mii_ioctl_data *mii;

	mii = (struct mii_ioctl_data *)(&ifr.ifr_ifru.ifru_data);
	mii->phy_id = portNumber;
	mii->reg_num = MIIReg;

	if (ioctl(fd, SIOCGMIIREG, &ifr) < 0)
	{
		perror("ioctl[SIOCGMIIPHY]");
		return GT_FALSE;
	}

	*value = (unsigned int)(mii->val_out & 0xffff);

	return GT_TRUE;
}

GT_BOOL MiiWrite (GT_QD_DEV* dev, unsigned int portNumber , unsigned int MIIReg,
		unsigned int value)
{
	struct mii_ioctl_data *mii;
	mii = (struct mii_ioctl_data *)(&ifr.ifr_ifru.ifru_data);
	mii->phy_id = portNumber;
	mii->reg_num = MIIReg;
	mii->val_in = value & 0xffff;

	if (ioctl(fd, SIOCSMIIREG, &ifr) < 0)
	{
		perror("ioctl[SIOCSMIIPHY]");
		return GT_FALSE;
	}

	return GT_TRUE;
}

/*
 *    Enable EgressMonitoring for the monitoredPort.
 *    With this setup, monitoringPort will receive every packet 
 *    which egressed from monitoredPort.
*/
GT_STATUS sampleEgressMonitor(GT_QD_DEV *dev,GT_LPORT monitoredPort, GT_LPORT monitoringPort)
{
    GT_STATUS status;
    GT_U16 pav;

    /*
     *    Enable EgressMonitoring for the monitoredPort.
    */
    pav = (1<<monitoringPort) || (1<<monitoredPort);

    if((status = gpavSetPAV(dev,monitoredPort, pav)) != GT_OK)
    {
        printf(("gpavSetPAV return Failed\n"));
        return status;
    }

    return GT_OK;
}

/*
 *    Enable Egress Monitoring and Ingress Monitoring for the monitoredPort.
 *    With this setup, monitoringPort will receive every packet 
 *    which is both from monitoredPort and to monitoredPort.
*/
GT_STATUS samplePortMonitor(GT_QD_DEV *dev,GT_LPORT monitoredPort, GT_LPORT monitoringPort)
{
    GT_STATUS status;
    GT_U16 pav;

    /*
     *    Enable Egress Monitoring for the monitoredPort.
    */
    pav = (1<<monitoringPort) || (1<<monitoredPort);

    if((status = gpavSetPAV(dev,monitoredPort, pav)) != GT_OK)
    {
        printf(("gpavSetPAV return Failed\n"));
        return status;
    }

    /*
     *    Enable Ingress Monitoring for the monitoredPort.
    */
    if((status = gpavSetIngressMonitor(dev,monitoredPort, GT_TRUE)) != GT_OK)
    {
        printf(("gpavSetIngressMonitor return Failed\n"));
        return status;
    }

    return GT_OK;
}


/*
 *    Disable Egress Monitoring and Ingress Monitoring for the monitoredPort.
*/
GT_STATUS sampleDisablePortMonitor(GT_QD_DEV *dev,GT_LPORT monitoredPort)
{
    GT_STATUS status;
    GT_U16 pav;

    /*
     *    Disable Egress Monitoring for the monitoredPort.
    */
    pav = (1<<monitoredPort);

    if((status = gpavSetPAV(dev,monitoredPort, pav)) != GT_OK)
    {
        printf(("gpavSetPAV return Failed\n"));
        return status;
    }

    /*
     *    Disable Ingress Monitoring for the monitoredPort.
    */
    if((status = gpavSetIngressMonitor(dev,monitoredPort, GT_FALSE)) != GT_OK)
    {
        printf(("gpavSetIngressMonitor return Failed\n"));
        return status;
    }

    return GT_OK;
}

	

GT_STATUS qdVlanEnable(void)
{
	GT_STATUS status;
	IN GT_LPORT  memPort3[]={4,5};
	IN GT_U8     memPort3Len =2;

	IN GT_LPORT  memPort4[]={3};
	IN GT_U8     memPort4Len =1;

	IN GT_LPORT  memPort5[]={3};
	IN GT_U8     memPort5Len =1;

	/* Port based vlan configuration */
	if((status=gvlnSetPortVlanPorts(dev, 3,memPort3,memPort3Len))==GT_OK)
	{
		//printf("DEBUG:Vlan for Port3 Enabled\n");
	}

	if((status=gvlnSetPortVlanPorts(dev, 4,memPort4,memPort4Len))==GT_OK)
	{
		//printf("DEBUG:Vlan for Port4 Enabled\n");
	}

	if((status=gvlnSetPortVlanPorts(dev, 5,memPort5,memPort5Len))==GT_OK)
	{
		//printf("DEBUG:Vlan for Port5 Enabled\n");
	}

	return GT_TRUE;
}

GT_STATUS qdVlanDisable(void)
{
	GT_STATUS status;
	IN GT_LPORT  memPort3[]={4,5};
	IN GT_U8     memPort3Len =2;

	IN GT_LPORT  memPort4[]={3,5};
	IN GT_U8     memPort4Len =2;

	IN GT_LPORT  memPort5[]={3,4};
	IN GT_U8     memPort5Len =2;

	/* Port based vlan configuration */
	if((status=gvlnSetPortVlanPorts(dev, 3,memPort3,memPort3Len))==GT_OK)
	{
		//printf("DEBUG:Vlan for Port3 Disabled\n");
	}

	if((status=gvlnSetPortVlanPorts(dev, 4,memPort4,memPort4Len))==GT_OK)
	{
		//printf("DEBUG:Vlan for Port4 Disabled\n");
	}

	if((status=gvlnSetPortVlanPorts(dev, 5,memPort5,memPort5Len))==GT_OK)
	{
		//printf("DEBUG:Vlan for Port5 Disabled\n");
	}

	return GT_TRUE;
}

GT_STATUS qdVTU_BusyWait()
{
    GT_STATUS status;
    GT_U16 tmpReg=0;
    GT_U16 max=100;
    do
    {
      status=gprtGetGlobalReg(dev,0x5, &tmpReg);
      usleep(1);
      if( --max == 0)
      	break;
    }while(((tmpReg >> 15)> 0) && (status == GT_OK));
    //printf("VTU wait %u (0x%04x)\n",max,tmpReg);
    return status;
}


GT_STATUS qdTagVlanApi_LAN(GT_U16 lanVID, GT_U16 lanPrio)
{
	GT_STATUS status;
	GT_VTU_ENTRY vtuEntry;
	GT_LPORT port;
	OUT GT_U16 i;
	GT_U16 tmpReg=0;
	
	//IN GT_LPORT  memPort3[]={4,5};
	//IN GT_U8     memPort3Len =2;
	//IN GT_LPORT  memPort4[]={3,5};
	//IN GT_U8     memPort4Len =2;
	IN GT_LPORT  memPort3[]={4,5};
	IN GT_U8     memPort3Len =2;
	IN GT_LPORT  memPort4[]={3};
	IN GT_U8     memPort4Len =1;
	IN GT_LPORT  memPort5[]={3};
	IN GT_U8     memPort5Len =1;
	
	
	/* Chip BUG, add dummy entry */
	gprtSetGlobalReg(dev, 0x3, 0x0);
	gprtSetGlobalReg(dev, 0x6, 0x1000);
	gprtSetGlobalReg(dev, 0x7, 0x0);

	gprtSetGlobalReg(dev, 0x8, 0x0);
	gprtSetGlobalReg(dev, 0x9, 0x0);
	gprtSetGlobalReg(dev, 0x5, 0xD000);
	qdVTU_BusyWait();
	/*
	 *    Clear VLAN ID Table
	 */
	if((status = gvtuFlush(dev)) != GT_OK)
	{
	    printf(("gvtuFlush returned fail.\n"));
	    return status;
	}
	qdVTU_BusyWait();
	
		
	/*
	 *    Add VLAN ID with Port 3,4 and CPU Port as members of the Vlan.
	 */
	gtMemSet(&vtuEntry,0,sizeof(GT_VTU_ENTRY));
	vtuEntry.DBNum = 0;
	vtuEntry.vid = lanVID;
	//vtuEntry.sid = 1;

	for(i=0; i<dev->numOfPorts; i++)
	{
		port = i;
		if(port==3)
			vtuEntry.vtuData.memberTagP[port] = MEMBER_EGRESS_TAGGED;
		else if((port== 5) || (port ==4))
			vtuEntry.vtuData.memberTagP[port] = MEMBER_EGRESS_UNTAGGED;
		else
			vtuEntry.vtuData.memberTagP[port] = NOT_A_MEMBER;
	}

	if((status = gvtuAddEntry(dev,&vtuEntry)) != GT_OK)
	{
		printf("gvtuAddEntry returned fail.\n");
		return status;
	}
	
	qdVTU_BusyWait();
	
	
	/*
	 * Egress unmodified, Normal network, Egress all unkown, Forwarding
	 * FID=0, allow packet forwarding to P3
	 * Force lanVID and Default lanPrio
	 * 802.1Q enabled in Fallbackmode, MapDA=1
	 */
	gprtSetEgressMode(dev, 5, GT_UNMODIFY_EGRESS);
	gprtSetEgressFlood(dev, 5, GT_BLOCK_EGRESS_NONE);
	gstpSetPortState(dev, 5, GT_PORT_FORWARDING);
	gvlnSetPortVlanPorts(dev, 5,memPort5,memPort5Len);
	gvlnSetPortVlanForceDefaultVID(dev, 5, GT_TRUE);
	gvlnSetPortVid(dev, 5, lanVID);
	gqosSetDefFPri(dev, 5, lanPrio);
	gvlnSetPortVlanDot1qMode(dev, 5, GT_FALLBACK);
	gprtSetMapDA(dev, 5, GT_TRUE);
	

	/* 
	 * Egress unmodified, Normal network, Egress unkown Multicast, Forwarding
	 * FID=0, allow packet forwarding to P4,P5
	 * 802.1Q enabled in Fallbackmode, MapDA=1
	 */ 
	gprtSetEgressMode(dev, 3, GT_UNMODIFY_EGRESS);
	gprtSetEgressFlood(dev, 3, GT_BLOCK_EGRESS_UNKNOWN_UNICAST);
	gstpSetPortState(dev, 3, GT_PORT_FORWARDING);
	gvlnSetPortVlanPorts(dev, 3,memPort3,memPort3Len);
	gvlnSetPortVlanForceDefaultVID(dev, 3, GT_FALSE);
	gvlnSetPortVid(dev, 3, 0x0);
	gqosSetDefFPri(dev, 3, 0x0);
	gvlnSetPortVlanDot1qMode(dev, 3, GT_FALLBACK);
	gprtSetMapDA(dev, 3, GT_TRUE);
	
	/* Egress unmodified, Normal network, Egress unkown Multicast, Forwarding
	 * FID=0, allow packet forwarding to P3
	 * 802.1Q enabled in Fallbackmode, MapDA=1
	 */ 
	gprtSetEgressMode(dev, 4, GT_UNMODIFY_EGRESS);
	gprtSetEgressFlood(dev, 4, GT_BLOCK_EGRESS_UNKNOWN_UNICAST);
	gstpSetPortState(dev, 4, GT_PORT_FORWARDING);
	gvlnSetPortVlanPorts(dev, 4, memPort4, memPort4Len);
	gvlnSetPortVlanForceDefaultVID(dev, 4, GT_FALSE);
	gvlnSetPortVid(dev, 4, 0x0);
	gqosSetDefFPri(dev, 4, 0x0);
	gvlnSetPortVlanDot1qMode(dev, 4, GT_FALLBACK);
	gprtSetMapDA(dev, 4, GT_TRUE);
	
#ifdef DEBUG
	printf("Getting VTU Entries\n");
	gprtSetGlobalReg(dev, 0x6, 0x0fff);
	gprtSetGlobalReg(dev, 0x5, 0xC000);
	qdVTU_BusyWait();
	status=gprtGetGlobalReg(dev,0x6, &tmpReg);
	printf("VTU 6: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x7, &tmpReg);
	printf("VTU 7: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x8, &tmpReg);
	printf("VTU 8: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x9, &tmpReg);
	printf("VTU 9: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x2, &tmpReg);
	printf("VTU 2: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x3, &tmpReg);
	printf("VTU 3: 0x%04X\n",tmpReg);
#endif	
	
	return GT_OK;
	
}

GT_STATUS qdTagVlanReg_LAN(GT_U16 lanVID, GT_U16 lanPrio)
{
	GT_U16 tmpReg=0;
	GT_STATUS status;
	
	/* Chip BUG, add dummy entry */
	gprtSetGlobalReg(dev, 0x3, 0x0);
	gprtSetGlobalReg(dev, 0x6, 0x1000);
	gprtSetGlobalReg(dev, 0x7, 0x0);
	gprtSetGlobalReg(dev, 0x8, 0x0);
	gprtSetGlobalReg(dev, 0x9, 0x0);
	gprtSetGlobalReg(dev, 0x5, 0xD000); 	// VTU Op
	qdVTU_BusyWait();
	
	//Clear VLAN ID Table
	if((status = gvtuFlush(dev)) != GT_OK)
	{
	    printf(("gvtuFlush returned fail.\n"));
	    return status;
	}
	qdVTU_BusyWait();
	
	//VTU for lanVID
	tmpReg=(0x1000 | (lanVID & 0x0fff));
	gprtSetGlobalReg(dev, 0x6, tmpReg);	// VID=lanVID, Valid=1
	gprtSetGlobalReg(dev, 0x7, 0x2333);	// P3 is member, egress P3 tagged
	gprtSetGlobalReg(dev, 0x8, 0x0311);	// P4,P5 are members, egress P4, P5 untagged
	gprtSetGlobalReg(dev, 0x5, 0xB000);	// VTU Op
	qdVTU_BusyWait();
	
	/* Port 5 register configuration */
	hwWritePortReg(dev, 5, 4, 0xf);	// Egress unmodified, Normal network, Egress all unkown, Forwarding
	//hwWritePortReg(dev, 5, 6, 0x18);	// FID=0, allow packet forwarding to P3,P4
	hwWritePortReg(dev, 5, 6, 0x08);	// FID=0, allow packet forwarding to P3
	tmpReg=(0x1000 | ((lanPrio & 0x7) << 13) | (lanVID & 0x0fff));
	hwWritePortReg(dev, 5, 7, tmpReg);	// Force lanVID and Default lanPrio
	hwWritePortReg(dev, 5, 8, 0x2480);	// 802.1Q enabled in Fallbackmode, MapDA=1

	/* Port 3 register configuration */
	hwWritePortReg(dev, 3, 4, 0xb);	// Egress unmodified, Normal network, Egress unkown Multicast, Forwarding
	hwWritePortReg(dev, 3, 6, 0x30);	// FID=0, allow packet forwarding to P4,P5
	hwWritePortReg(dev, 3, 8, 0x2480);	// 802.1Q enabled in Fallbackmode, MapDA=1

	/* Port 4 register configuration */
	hwWritePortReg(dev, 4, 4, 0xb);	// Egress unmodified, Normal network, Egress unkown Multicast, Forwarding
	//hwWritePortReg(dev, 4, 6, 0x28);	// FID=0, allow packet forwarding to P3,P5
	hwWritePortReg(dev, 4, 6, 0x08);	// FID=0, allow packet forwarding to P3
	hwWritePortReg(dev, 4, 8, 0x2480);	// 802.1Q enabled in Fallbackmode, MapDA=1
	
	
#ifdef DEBUG	
	printf("Getting VTU Entries\n");
	gprtSetGlobalReg(dev, 0x6, 0x0fff);
	gprtSetGlobalReg(dev, 0x5, 0xC000);
	qdVTU_BusyWait();
	status=gprtGetGlobalReg(dev,0x6, &tmpReg);
	printf("VTU 6: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x7, &tmpReg);
	printf("VTU 7: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x8, &tmpReg);
	printf("VTU 8: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x9, &tmpReg);
	printf("VTU 9: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x2, &tmpReg);
	printf("VTU 2: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x3, &tmpReg);
	printf("VTU 3: 0x%04X\n",tmpReg);
#endif	

	return GT_OK;
}

GT_STATUS qdTagVlanApi_LAN_PC(GT_U16 lanVID, GT_U16 lanPrio,GT_U16 pcVID, GT_U16 pcPrio)
{
	GT_STATUS status;
	GT_VTU_ENTRY vtuEntry;
	GT_LPORT port;
	OUT GT_U16 i;
	GT_U16 tmpReg=0;
	
	IN GT_LPORT  memPort3[]={4,5};
	IN GT_U8     memPort3Len =2;
	IN GT_LPORT  memPort4[]={3};
	IN GT_U8     memPort4Len =1;
	IN GT_LPORT  memPort5[]={3};
	IN GT_U8     memPort5Len =1;
	
	
	/* Chip BUG, add dummy entry */
	gprtSetGlobalReg(dev, 0x3, 0x0);
	gprtSetGlobalReg(dev, 0x6, 0x1000);
	gprtSetGlobalReg(dev, 0x7, 0x0);

	gprtSetGlobalReg(dev, 0x8, 0x0);
	gprtSetGlobalReg(dev, 0x9, 0x0);
	gprtSetGlobalReg(dev, 0x5, 0xD000);
	qdVTU_BusyWait();
	/*
	 *    Clear VLAN ID Table
	 */
	if((status = gvtuFlush(dev)) != GT_OK)
	{
	    printf(("gvtuFlush returned fail.\n"));
	    return status;
	}
	qdVTU_BusyWait();
	
		
	/*
	 *    Add VLAN ID with Port 3,4 and CPU Port as members of the Vlan.
	 */
	gtMemSet(&vtuEntry,0,sizeof(GT_VTU_ENTRY));
	vtuEntry.DBNum = 0;
	vtuEntry.vid = lanVID;
	//vtuEntry.sid = 1;

	for(i=0; i<dev->numOfPorts; i++)
	{
		port = i;
		if((port==3) || (port ==4))
			vtuEntry.vtuData.memberTagP[port] = MEMBER_EGRESS_TAGGED;
		else if(port== 5)
			vtuEntry.vtuData.memberTagP[port] = MEMBER_EGRESS_UNTAGGED;
		else
			vtuEntry.vtuData.memberTagP[port] = NOT_A_MEMBER;
	}

	if((status = gvtuAddEntry(dev,&vtuEntry)) != GT_OK)
	{
		printf("gvtuAddEntry returned fail.\n");
		return status;
	}
	
	qdVTU_BusyWait();
	
	/*
	 *    Add VLAN ID with Port 3,4 and CPU Port as members of the Vlan.
	 */
	gtMemSet(&vtuEntry,0,sizeof(GT_VTU_ENTRY));
	vtuEntry.DBNum = 0;
	vtuEntry.vid = pcVID;
	//vtuEntry.sid = 1;

	for(i=0; i<dev->numOfPorts; i++)
	{
		port = i;
		if(port==3)
			vtuEntry.vtuData.memberTagP[port] = MEMBER_EGRESS_TAGGED;
		else if(port ==4)
			vtuEntry.vtuData.memberTagP[port] = MEMBER_EGRESS_UNTAGGED;
		else
			vtuEntry.vtuData.memberTagP[port] = NOT_A_MEMBER;
	}

	if((status = gvtuAddEntry(dev,&vtuEntry)) != GT_OK)
	{
		printf("gvtuAddEntry returned fail.\n");
		return status;
	}
	
	qdVTU_BusyWait();
	
	
	/*
	 * Egress unmodified, Normal network, Egress all unkown, Forwarding
	 * FID=0, allow packet forwarding to P3
	 * Force lanVID and Default lanPrio
	 * 802.1Q enabled in Fallbackmode, MapDA=1
	 */
	gprtSetEgressMode(dev, 5, GT_UNMODIFY_EGRESS);
	gprtSetEgressFlood(dev, 5, GT_BLOCK_EGRESS_NONE);
	gstpSetPortState(dev, 5, GT_PORT_FORWARDING);
	gvlnSetPortVlanPorts(dev, 5,memPort5,memPort5Len);
	gvlnSetPortVlanForceDefaultVID(dev, 5, GT_TRUE);
	gvlnSetPortVid(dev, 5, lanVID);
	gqosSetDefFPri(dev, 5, lanPrio);
	gvlnSetPortVlanDot1qMode(dev, 5, GT_FALLBACK);
	gprtSetMapDA(dev, 5, GT_TRUE);
	

	/* 
	 * Egress unmodified, Normal network, Egress unkown Multicast, Forwarding
	 * FID=0, allow packet forwarding to P4,P5
	 * 802.1Q enabled in Fallbackmode, MapDA=1
	 */ 
	gprtSetEgressMode(dev, 3, GT_UNMODIFY_EGRESS);
	gprtSetEgressFlood(dev, 3, GT_BLOCK_EGRESS_UNKNOWN_UNICAST);
	gstpSetPortState(dev, 3, GT_PORT_FORWARDING);
	gvlnSetPortVlanPorts(dev, 3,memPort3,memPort3Len);
	gvlnSetPortVlanForceDefaultVID(dev, 3, GT_FALSE);
	gvlnSetPortVid(dev, 3, 0x0);
	gqosSetDefFPri(dev, 3, 0x0);
	gvlnSetPortVlanDot1qMode(dev, 3, GT_FALLBACK);
	gprtSetMapDA(dev, 3, GT_TRUE);
	
	/* Egress unmodified, Normal network, Egress unkown Multicast, Forwarding
	 * FID=0, allow packet forwarding to P3
	 * pcVID and Default pcPrio
	 * 802.1Q enabled in Fallbackmode, MapDA=1
	 */ 
	gprtSetEgressMode(dev, 4, GT_UNMODIFY_EGRESS);
	gprtSetEgressFlood(dev, 4, GT_BLOCK_EGRESS_UNKNOWN_UNICAST);
	gstpSetPortState(dev, 4, GT_PORT_FORWARDING);
	gvlnSetPortVlanPorts(dev, 4, memPort4, memPort4Len);
	gvlnSetPortVlanForceDefaultVID(dev, 4, GT_FALSE);
	gvlnSetPortVid(dev, 4, pcVID);
	gqosSetDefFPri(dev, 4, pcPrio);
	gvlnSetPortVlanDot1qMode(dev, 4, GT_FALLBACK);
	gprtSetMapDA(dev, 4, GT_TRUE);
	
#ifdef DEBUG	
	printf("Getting VTU Entries\n");
	gprtSetGlobalReg(dev, 0x6, 0xfff);
	gprtSetGlobalReg(dev, 0x5, 0xC000);
	qdVTU_BusyWait();
	status=gprtGetGlobalReg(dev,0x6, &tmpReg);
	printf("VTU 6: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x7, &tmpReg);
	printf("VTU 7: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x8, &tmpReg);
	printf("VTU 8: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x9, &tmpReg);
	printf("VTU 9: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x2, &tmpReg);
	printf("VTU 2: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x3, &tmpReg);
	printf("VTU 3: 0x%04X\n",tmpReg);
	
	gprtSetGlobalReg(dev, 0x5, 0xC000);
	qdVTU_BusyWait();
	status=gprtGetGlobalReg(dev,0x6, &tmpReg);
	printf("VTU 6: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x7, &tmpReg);
	printf("VTU 7: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x8, &tmpReg);
	printf("VTU 8: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x9, &tmpReg);
	printf("VTU 9: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x2, &tmpReg);
	printf("VTU 2: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x3, &tmpReg);
	printf("VTU 3: 0x%04X\n",tmpReg);
#endif	
	return GT_OK;
}

GT_STATUS qdTagVlanReg_LAN_PC(GT_U16 lanVID, GT_U16 lanPrio, GT_U16 pcVID, GT_U16 pcPrio)
{
	GT_U16 tmpReg=0;
	GT_STATUS status;

	/* Chip BUG, add dummy entry */
	gprtSetGlobalReg(dev, 0x3, 0x0);
	gprtSetGlobalReg(dev, 0x6, 0x1000);
	gprtSetGlobalReg(dev, 0x7, 0x0);
	gprtSetGlobalReg(dev, 0x8, 0x0);
	gprtSetGlobalReg(dev, 0x9, 0x0);
	gprtSetGlobalReg(dev, 0x5, 0xD000);	// VTU Op
	qdVTU_BusyWait();

	//Clear VLAN ID Table
	if((status = gvtuFlush(dev)) != GT_OK)
	{
		printf(("gvtuFlush returned fail.\n"));
		return status;
	}
	qdVTU_BusyWait();
	//VTU for lanVID
	tmpReg=(0x1000 | (lanVID & 0x0fff));
	gprtSetGlobalReg(dev, 0x6, tmpReg);	// VID=lanVID, Valid=1
	gprtSetGlobalReg(dev, 0x7, 0x2333);	// P3 is member, egress P3 tagged
	gprtSetGlobalReg(dev, 0x8, 0x0312);	// P4,P5 are members, egress P4 tagged, P5 untagged
	gprtSetGlobalReg(dev, 0x5, 0xB000);   	// VTU Op
	qdVTU_BusyWait();
	//VTU for pcVID
	tmpReg=(0x1000 | (pcVID & 0x0fff));
	gprtSetGlobalReg(dev, 0x6, tmpReg);	// VID=pcVID, Valid=1
	gprtSetGlobalReg(dev, 0x7, 0x2333);	// P3 is member, egress P3 tagged
	gprtSetGlobalReg(dev, 0x8, 0x0331);	// P4 is member, egress P4 untagged
	gprtSetGlobalReg(dev, 0x5, 0xB000);	// VTU Op
	qdVTU_BusyWait();
	/* Port 5 register configuration */
	hwWritePortReg(dev, 5, 4, 0x100f);	// Egress untagged, Normal network, Egress all unkown, Forwarding
	hwWritePortReg(dev, 5, 6, 0x08);	// FID=0, allow packet forwarding to P3
	tmpReg=(0x1000 | ((lanPrio & 0x7) << 13) | (lanVID & 0x0fff)); // Force lanVID and Default lanPrio
	hwWritePortReg(dev, 5, 7, tmpReg);	
	hwWritePortReg(dev, 5, 8, 0x2480);	// 802.1Q enabled in Fallbackmode, MapDA=1

	/* Port 3 register configuration */
	hwWritePortReg(dev, 3, 4, 0xb); 	// Egress unmodified, Normal network, Egress unkown Multicast, Forwarding
	hwWritePortReg(dev, 3, 6, 0x30); 	// FID=0, allow packet forwarding to P4,P5
	hwWritePortReg(dev, 3, 8, 0x2480); 	// 802.1Q enabled in Fallbackmode, MapDA=1

	/* Port 4 register configuration */
	hwWritePortReg(dev, 4, 4, 0xb);	// Egress unmodified, Normal network, Egress unkown Multicast, Forwarding
	hwWritePortReg(dev, 4, 6, 0x08);	// FID=0, allow packet forwarding to P3
	tmpReg=(0x0000 | ((pcPrio & 0x7) << 13) | (pcVID & 0x0fff)); // pcVID and Default pcPrio
	hwWritePortReg(dev, 4, 7, tmpReg);	
	hwWritePortReg(dev, 4, 8, 0x2480);	// 802.1Q enabled in Fallbackmode, MapDA=1

#ifdef DEBUG	
	printf("Getting VTU Entries\n");
	gprtSetGlobalReg(dev, 0x6, 0xfff);
	gprtSetGlobalReg(dev, 0x5, 0xC000);
	qdVTU_BusyWait();
	status=gprtGetGlobalReg(dev,0x6, &tmpReg);
	printf("VTU 6: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x7, &tmpReg);
	printf("VTU 7: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x8, &tmpReg);
	printf("VTU 8: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x9, &tmpReg);
	printf("VTU 9: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x2, &tmpReg);
	printf("VTU 2: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x3, &tmpReg);
	printf("VTU 3: 0x%04X\n",tmpReg);

	gprtSetGlobalReg(dev, 0x5, 0xC000);
	qdVTU_BusyWait();
	status=gprtGetGlobalReg(dev,0x6, &tmpReg);
	printf("VTU 6: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x7, &tmpReg);
	printf("VTU 7: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x8, &tmpReg);
	printf("VTU 8: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x9, &tmpReg);
	printf("VTU 9: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x2, &tmpReg);
	printf("VTU 2: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x3, &tmpReg);
	printf("VTU 3: 0x%04X\n",tmpReg);
#endif	

	return GT_OK;
}

GT_STATUS qdTagVlanReg_PC(GT_U16 pcVID, GT_U16 pcPrio)
{
	GT_U16 tmpReg=0;
	GT_STATUS status;

	/* Chip BUG, add dummy entry */
	gprtSetGlobalReg(dev, 0x3, 0x0);
	gprtSetGlobalReg(dev, 0x6, 0x1000);
	gprtSetGlobalReg(dev, 0x7, 0x0);
	gprtSetGlobalReg(dev, 0x8, 0x0);
	gprtSetGlobalReg(dev, 0x9, 0x0);
	gprtSetGlobalReg(dev, 0x5, 0xD000);	// VTU Op
	qdVTU_BusyWait();

	//Clear VLAN ID Table
	if((status = gvtuFlush(dev)) != GT_OK)
	{
	    printf(("gvtuFlush returned fail.\n"));
	    return status;
	}
	qdVTU_BusyWait();

	//VTU for pcVID
	tmpReg=(0x1000 | (pcVID & 0x0fff));
	gprtSetGlobalReg(dev, 0x6, tmpReg);	// VID=pcVID, Valid=1
	gprtSetGlobalReg(dev, 0x7, 0x2333);	// P3 is member, egress P3 tagged
	gprtSetGlobalReg(dev, 0x8, 0x0331);	// P4 is member, egress P4 untagged
	gprtSetGlobalReg(dev, 0x5, 0xB000);	// VTU Op
	qdVTU_BusyWait();

	/* Port 5 register configuration */
	hwWritePortReg(dev, 5, 4, 0x100f);	// Egress untagged, Normal network, Egress all unkown, Forwarding
	hwWritePortReg(dev, 5, 6, 0x08);	// FID=0, allow packet forwarding to P3
	hwWritePortReg(dev, 5, 7, 0x0);
	hwWritePortReg(dev, 5, 8, 0x2480);	// 802.1Q enabled in Fallbackmode, MapDA=1

	/* Port 3 register configuration */
	hwWritePortReg(dev, 3, 4, 0xb); 	// Egress unmodified, Normal network, Egress unkown Multicast, Forwarding
	hwWritePortReg(dev, 3, 6, 0x30); 	// FID=0, allow packet forwarding to P4,P5
	hwWritePortReg(dev, 3, 8, 0x2480); 	// 802.1Q enabled in Fallbackmode, MapDA=1

	/* Port 4 register configuration */
	hwWritePortReg(dev, 4, 4, 0xb);	// Egress unmodified, Normal network, Egress unkown Multicast, Forwarding
	hwWritePortReg(dev, 4, 6, 0x08);	// FID=0, allow packet forwarding to P3
	tmpReg=(0x0000 | ((pcPrio & 0x7) << 13) | (pcVID & 0x0fff)); // pcVID and Default pcPrio
	hwWritePortReg(dev, 4, 7, tmpReg);
	hwWritePortReg(dev, 4, 8, 0x2480);	// 802.1Q enabled in Fallbackmode, MapDA=1

#ifdef DEBUG
	printf("Getting VTU Entries\n");
	gprtSetGlobalReg(dev, 0x6, 0xfff);
	gprtSetGlobalReg(dev, 0x5, 0xC000);
	qdVTU_BusyWait();
	status=gprtGetGlobalReg(dev,0x6, &tmpReg);
	printf("VTU 6: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x7, &tmpReg);
	printf("VTU 7: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x8, &tmpReg);
	printf("VTU 8: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x9, &tmpReg);
	printf("VTU 9: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x2, &tmpReg);
	printf("VTU 2: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x3, &tmpReg);
	printf("VTU 3: 0x%04X\n",tmpReg);

	gprtSetGlobalReg(dev, 0x5, 0xC000);
	qdVTU_BusyWait();
	status=gprtGetGlobalReg(dev,0x6, &tmpReg);
	printf("VTU 6: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x7, &tmpReg);
	printf("VTU 7: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x8, &tmpReg);
	printf("VTU 8: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x9, &tmpReg);
	printf("VTU 9: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x2, &tmpReg);
	printf("VTU 2: 0x%04X\n",tmpReg);
	status=gprtGetGlobalReg(dev,0x3, &tmpReg);
	printf("VTU 3: 0x%04X\n",tmpReg);
#endif

	return GT_OK;
}

GT_STATUS qdTagVlanRegDisable(void)
{
  gvtuFlush(dev);
  qdVTU_BusyWait();
  
  hwWritePortReg(dev, 5, 4, 0xf);	// Egress unmodified, Normal network, Egress all unkown, Forwarding
  hwWritePortReg(dev, 5, 6, 0x18);	// FID=0, allow packet forwarding to P3,P4
  hwWritePortReg(dev, 5, 7, 0x0);	// Reset VID to default
  hwWritePortReg(dev, 5, 8, 0x2080);	// Jumbo Frames, 802.1Q disabled, MapDA=1

  /* Port 3 register configuration */
  hwWritePortReg(dev, 3, 4, 0xf); 	// Egress unmodified, Normal network, Egress unkown Multicast, Forwarding
  hwWritePortReg(dev, 3, 6, 0x30); 	// FID=0, allow packet forwarding to P4,P5
  hwWritePortReg(dev, 3, 8, 0x2080); 	// Jumbo Frames, 802.1Q disabled, MapDA=1

  /* Port 4 register configuration */
  hwWritePortReg(dev, 4, 4, 0xf);	// Egress unmodified, Normal network, Egress unkown Multicast, Forwarding
  hwWritePortReg(dev, 4, 6, 0x28);	// FID=0, allow packet forwarding to P3,P5
  hwWritePortReg(dev, 4, 7, 0x0);	// Reset VID to default
  hwWritePortReg(dev, 4, 8, 0x2080);	// Jumbo Frames, 802.1Q disabled, MapDA=1

  return GT_OK;
}


GT_STATUS SetVlanConfig(VlanInfo *pstVlanArg)
{
  if((pstVlanArg->lanVid > 0) && (pstVlanArg->pcVid > 0))
  {
    return qdTagVlanReg_LAN_PC(pstVlanArg->lanVid, pstVlanArg->lanPri, pstVlanArg->pcVid, pstVlanArg->pcPri);
  }
  if((pstVlanArg->lanVid > 0) && (pstVlanArg->pcVid == 0))
  {
    return qdTagVlanReg_LAN(pstVlanArg->lanVid, pstVlanArg->lanPri);
  }
  if((pstVlanArg->lanVid == 0) && (pstVlanArg->pcVid > 0))
  {
    return qdTagVlanReg_PC(pstVlanArg->pcVid, pstVlanArg->pcPri);
  }
  if((pstVlanArg->lanVid == 0) && (pstVlanArg->pcVid == 0))
  {
    return qdTagVlanRegDisable();
  }
  return GT_FAIL;
}




/*
 *  Initialize the QuarterDeck. This should be done in BSP driver init routine.
 *    Since BSP is not combined with QuarterDeck driver, we are doing here.
 */
#define LAN_LINK	"/sys/devices/platform/enet.0/lanLink"
#define PC_LINK		"/sys/devices/platform/enet.0/pcLink"

static void sysfs_write(const char *path, const char *s)
{
	int len;
	int fd = open(path, O_WRONLY);

	if (fd < 0) {
		printf("Error opening %s: %s\n", path, strerror(errno));
		return;
	}

	len = write(fd, s, strlen(s));
	if (len < 0) {
		printf("Error writing to %s: %s\n", path, strerror(errno));
	}

	close(fd);
}

GT_STATUS qdStart(int cpuPort, int useQdSim, int devId, VlanInfo *pstVlanArg) /* devId is used for simulator only */
{
	GT_STATUS status;
	OUT GT_U16 data_reg;
#ifdef DEBUG
	OUT GT_U16 j,k;
	OUT GT_U16 phyreg[0x20];
	OUT GT_U16 phyreg1[0x20], globalreg[0x20],globalreg2[0x20],globalreg3[0x20];
	OUT GT_U32 switchreg[0x20];
	OUT GT_U32 switchreg1[0x20];
	OUT GT_U32 switchreg2[0x20];
#endif
	
	OUT GT_BOOL	link3status = GT_FALSE,link4status = GT_FALSE;
	OUT GT_BOOL	oldlink3status = GT_FALSE, oldlink4status = GT_FALSE;
	/*
	 *  Register all the required functions to QuarterDeck Driver.
	 */
	memset((char*)&cfg,0,sizeof(GT_SYS_CONFIG));
	memset((char*)&diagDev,0,sizeof(GT_QD_DEV));

	cfg.BSPFunctions.readMii   = MiiRead;
	cfg.BSPFunctions.writeMii  = MiiWrite;
	cfg.BSPFunctions.semCreate = NULL;
	cfg.BSPFunctions.semDelete = NULL;
	cfg.BSPFunctions.semTake   = NULL;
	cfg.BSPFunctions.semGive   = NULL;
	MiiInit(dev);

	cfg.initPorts = GT_TRUE;    /* Set switch ports to Forwarding mode. If GT_FALSE, use Default Setting. */
	cfg.cpuPortNum = cpuPort;
#ifdef MANUAL_MODE    /* not defined. this is only for sample */
	/* user may want to use this mode when there are two QD switchs on the same MII bus. */
	cfg.mode.scanMode = SMI_MANUAL_MODE;    /* Use QD located at manually defined base addr */
	cfg.mode.baseAddr = 0x10;    /* valid value in this case is either 0 or 0x10 */
	printf("DEBUG:qdStart start MANUAL_MODE\n");
#else
#ifdef MULTI_ADDR_MODE
	cfg.mode.scanMode = SMI_MULTI_ADDR_MODE;    /* find a QD in indirect access mode */
	cfg.mode.baseAddr = 1;        /* this is the phyAddr used by QD family device.
					 Valid value are 1 ~ 31.*/
	printf("DEBUG:qdStart start MULTI_ADDR_MODE\n");
#else
	cfg.mode.scanMode = SMI_AUTO_SCAN_MODE;    /* Scan 0 or 0x10 base address to find the QD */
	cfg.mode.baseAddr = 0;
#endif
#endif

	if((status=qdLoadDriver(&cfg, dev)) != GT_OK)
	{
		printf("DEBUG:qdLoadDriver return Failed\n");
		return status;
	}

	gprtSetGlobal2Reg(dev, 0x0, 0x1f);
	gprtSetGlobal2Reg(dev, 0x1, 0x1f);
	gprtSetGlobal2Reg(dev, 0x1C, 0x1f);

	/*
	 *  start the QuarterDeck
	 */

	if((status=sysEnable(dev)) != GT_OK)
	{
		printf("DEBUG:sysConfig return Failed\n");
		return status;
	}

	/* Configure TX/RX clock delay */
	if((status=gpcsSetRGMIITimingDelay(dev, 5, GT_TRUE,GT_FALSE))!=GT_OK)
	{
		printf("DEBUG: Error - gprt gpcsSetRGMIITimingDelay Port 5 \n");
	}

	if((status=gprtGetGlobalReg(dev, 0x1A ,&data_reg))==GT_OK)
	{
		data_reg &= 0xff5F;
		gprtSetGlobalReg(dev, 0x1A ,data_reg);
		gprtGetGlobalReg(dev, 0x1A ,&data_reg);
	}

	if((status=gprtPortPowerDown(dev, 3, GT_FALSE))!=GT_OK)
	{
		printf("DEBUG:Error - gprt PowerUp Port 3 \n");
	}

	if((status=gprtPortPowerDown(dev, 4, GT_FALSE))!=GT_OK)
	{
		printf("DEBUG:Error - gprt PowerUp Port 4 \n");
	}
	
	if((status=gprtSetLinkTimeWorkaround(dev, 4))!=GT_OK)
	{
		printf("DEBUG:Error - gprtSetLinkTimeWorkaround set \n");
	}

	SetVlanConfig(pstVlanArg);

#ifdef DEBUG
	printf("-----------PHY Reg Dump----------------------\n");
	printf("|Address\t |Port3\t |Port4\t |\n");
	printf("---------------------------------------------\n");
	for(j=0; j <=0x1F; j++ )
	{
		if((status=gprtGetPhyReg(dev, 3, j, &phyreg[j] ))==GT_OK)
		{
			if((status=gprtGetPhyReg(dev, 4, j, &phyreg1[j] ))==GT_OK)
			{
				printf("[%x]\t\t %x\t %x\t \n",j, phyreg[j], phyreg1[j]);
			}
		}

	}
	printf("---------------------------------------------\n\n\n");

	printf("------------Switch Port Reg Dump--------------\n");
	printf("|Address\t |Port3\t |Port4\t |Port5\t |\n");
	printf("----------------------------------------------\n");
	for(k=0; k<=0x1F; k++ )
	{
		gsysReadMiiReg(dev, 0x13, k,&switchreg[k]);
		gsysReadMiiReg(dev, 0x14, k,&switchreg1[k]);
		gsysReadMiiReg(dev, 0x15, k,&switchreg2[k]);
		printf("[%x]\t\t %lx\t %lx\t %lx\t \n",k,switchreg[k], switchreg1[k], switchreg2[k]);
	}
	printf("-----------------------------------------------\n");

	printf("------------Global Reg Dump--------------------\n");
	printf("|Address\t |Glob\t |Glob2\t |Glob3\t |\n");
	printf("----------------------------------------------\n");
	for(k=0; k<=0x1F; k++ )
	{
		gprtGetGlobalReg(dev, k,&globalreg[k]);
		gprtGetGlobal2Reg(dev, k,&globalreg2[k]);
		gprtGetGlobal3Reg(dev, k,&globalreg3[k]);
		printf("[%x]\t\t %x\t %x\t %x\t \n",k,globalreg[k], globalreg2[k], globalreg3[k]);
	}
	printf("----------------------------------------------\n");
#endif
	while(1)
	{
		sleep(1);
		gprtGetPhyLinkStatus(dev, 3, &link3status );
		gprtGetPhyLinkStatus(dev, 4, &link4status );

		if(link3status != oldlink3status)
		{
			if(link3status){
				if(link4status){
					sysfs_write(PC_LINK, "0");
					oldlink4status = link4status;
					gprtPortPowerDown(dev, 4, GT_TRUE);
					sleep(1);
					gprtPortPowerDown(dev, 4, GT_FALSE);
				}
				sysfs_write(LAN_LINK, "1");
				//qdVlanEnable();
				//qdTagVlan();
				//qdVlanEnableGeneral(lanVID,lanPrio,pcVID,pcPrio);
			}
			else{
				sysfs_write(LAN_LINK, "0");
				//qdVlanDisable();
				//qdVlanDisableGeneral();
				if(link4status){
					oldlink4status = 0;
					link4status = 0;
					gprtPortPowerDown(dev, 4, GT_TRUE);
					sleep(1);
					gprtPortPowerDown(dev, 4, GT_FALSE);
					sleep(1);
					//sysfs_write(PC_LINK, "1");
				}
			}
			oldlink3status = link3status;
		}
		if ((link4status != oldlink4status) && (link3status == 0))
		{
			if(link4status){
				sysfs_write(PC_LINK, "1");
			}
			else {

				sysfs_write(PC_LINK, "0");
			}
			oldlink4status = link4status;
		}

	}
	return GT_OK;
}

void sig_handler(int signo)
{
	GT_STATUS status;

	if (signo == SIGUSR1)
	{
		printf("received SIGUSR1, Power down Port 3 and Port 4...\n");

		if((status=gprtPortPowerDown(dev, 3, GT_TRUE))!=GT_OK)
		{
			printf("DEBUG:Error - gprt Power Down Port 3 \n");
		}

		if((status=gprtPortPowerDown(dev, 4, GT_TRUE))!=GT_OK)
		{
			printf("DEBUG:Error - gprt Power Down Port 4 \n");
		}
		printf("Done.\n");
	}
}

/*
 * trim: get rid of trailing and leading whitespace...
 *       ...including the annoying "\n" from fgets()
 */
char * trim (char * s)
{
	/* Initialize start, end pointers */
	char *s1 = s, *s2 = &s[strlen (s) - 1];

	/* Trim and delimit right side */
	while ( (isspace (*s2)) && (s2 >= s1) )
		s2--;
	*(s2+1) = '\0';

	/* Trim left side */
	while ( (isspace (*s1)) && (s1 < s2) )
		s1++;

	/* Copy finished string */
	strcpy (s, s1);
	return s;
}

void parse_config (VlanInfo *pstVlanArg, char *pVlanConfigFile)
{
	char *s, buff[256];
	FILE *fp = fopen (pVlanConfigFile, "r");

	if (fp == NULL || pstVlanArg == NULL)
	{
		printf("Unable to read VLAN config file %s\n", pVlanConfigFile);
		return;
	}

	/* Read next line */
	while ((s = fgets (buff, sizeof buff, fp)) != NULL)
	{
		/* Skip blank lines and comments */
		if (buff[0] == '\n' || buff[0] == '#')
			continue;

		/* Parse vlan property name/value pair from line */
		char name[MAXLEN], value[MAXLEN];
		s = strtok (buff, "=");
		if (s==NULL)
			continue;
		else
			strncpy (name, s, MAXLEN);
		s = strtok (NULL, "=");
		if (s==NULL)
			continue;
		else
			strncpy (value, s, MAXLEN);
		trim (value);

		/* Copy into correct entry in parameters struct */
		if (strcmp(name, PROPERTY_VLAN_TYPE)==0)
			pstVlanArg->vlan = atoi(value);
		else if (strcmp(name, PROPERTY_VLAN_LAN_VID)==0)
			pstVlanArg->lanVid = atoi(value);
		else if (strcmp(name, PROPERTY_VLAN_LAN_PRIO)==0)
			pstVlanArg->lanPri = atoi(value);
		else if (strcmp(name, PROPERTY_VLAN_PC_VID)==0)
			pstVlanArg->pcVid = atoi(value);
		else if (strcmp(name, PROPERTY_VLAN_PC_PRIO)==0)
			pstVlanArg->pcPri = atoi(value);
		else
			printf ("WARNING: %s/%s: Unknown name/value pair!\n",
					name, value);
	}

	/* Close file */
	fclose (fp);
}

void display_usage( void )
{
	puts( "vlan - enables vlan on PC or LAN port" );
	/* ... */
	exit( EXIT_FAILURE );
}

int main (int argc, char *argv[])
{
	int cpuPort = 5;
	int devId   = 0;
	int useQdSim = 0;
	GT_STATUS status;
	VlanInfo stVlanArg;
	char VlanConfigFile[MAXLEN+1];
#ifdef DEBUG
	freopen("/data/log/dsdt.log", "a", stdout);
	freopen("/data/log/dsdt.log", "a", stderr);
	setbuf(stdout, NULL);
	setbuf(stderr, NULL);
#endif
#if 0
	opt = getopt( argc, argv, optString );
	stVlanArg.pcVid = 0;
	stVlanArg.pcPri = 0;
	stVlanArg.lanVid = 0;
	stVlanArg.lanPri = 0;
	while( opt != -1 ) {
		switch( opt ) {
			case 'p':
				stVlanArg.pcVid = atoi(optarg);
				printf("PC VID %s\n", optarg);
				break;
			case 'q':
				stVlanArg.pcPri = atoi(optarg);
				printf("PC Pri %s\n", optarg);
				break;

			case 'l':
				stVlanArg.lanVid = atoi(optarg);
				printf("LAN VID %s\n", optarg);
				break;
			case 'm':
				stVlanArg.lanPri = atoi(optarg);
				printf("LAN Pri %s\n", optarg);
				break;

			case 'h':
			case '?':
				display_usage();
				break;

			default:
				break;
		}

		opt = getopt( argc, argv, optString );
	}
#endif
	stVlanArg.vlan = 0;
	stVlanArg.pcVid = 0;
	stVlanArg.pcPri = 0;
	stVlanArg.lanVid = 0;
	stVlanArg.lanPri = 0;
	if ( argc != 2 ) /* argc should be 2 for vlan settings */
	{
		/* We print argv[0] assuming it is the program name */
		printf( "VLAN config file missing\n");
	}
	else
	{
		strncpy(VlanConfigFile, argv[1], MAXLEN);
		printf("VLAN config file name - %s\n", VlanConfigFile);
	}

	parse_config(&stVlanArg, VlanConfigFile);
	printf("Values for vlan Config are..\n");
	printf("vlan = %d\nlanVid=%d\nlanPrio=%d\npcVid=%d\npcPrio=%d\n",
			stVlanArg.vlan, stVlanArg.lanVid, stVlanArg.lanPri, stVlanArg.pcVid, stVlanArg.pcPri);
	if (signal(SIGUSR1, sig_handler) == SIG_ERR)
		printf("\ncan't catch SIGUSR1\n");

	if ((status=qdStart(cpuPort, useQdSim, devId, &stVlanArg)) !=GT_OK)
	{
		printf("DEBUG:QuarterDeck has been Failed.\n");
		return status;
	}
	return 0;
}
