package maxwell.util;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.UnknownHostException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.Writer;
import java.util.Scanner;
import android.util.Log;
import java.util.Properties;

public final class Utils {

    public static final String TAG = "framework-custom-Utils";

    public static final String VENDOR_DIR = "/vendor_gigaset/";
    
    public static final String PROVISIONING_ID = "provisioning_id";

    public static final String PROVISIONING_ID_TXT_FILE = "provisioning_id.txt";

    public static final String PROPERTY_VARIANTID = "variantid";

    public static final String DEFAULT_VARIANT_ID = "1";

    public static final String ELMEG_VARIANT_ID = "2";

    public static final String ELMEG_STRING = "elmeg";

    public static final String PROPERTIES_FILE = "/fotatemp/maxwell.properties";

    /* Get product variant id from file
     */
    public static String getVariantID() {
        Log.d(TAG, "Load provisioningID from " + PROVISIONING_ID);
        String provisioningID = getProvisioningID(PROVISIONING_ID, ""); //load provisioning id from "provisioning_id" file
	if (provisioningID.equals("")) {
            Log.d(TAG, "Load provisioningID from " + PROVISIONING_ID_TXT_FILE); //load provisioning id from "provisioning_id.txt" file
            provisioningID = getProvisioningID(PROVISIONING_ID_TXT_FILE, "");
            if(provisioningID.equals("")) {
                Log.d(TAG, "Load provisioningID Maxwell.Properties File "); //load provisioning id from "maxwell.properties" file
                return getPropertyValue(PROPERTY_VARIANTID, DEFAULT_VARIANT_ID);
            }
        }
        return provisioningID;
    }

    public static final String getProvisioningID(String provIdFile, String defaultValue) {
        FileInputStream fs = null;
        try {
            String vendorPartitionValue = defaultValue;
            File f = new File(VENDOR_DIR, provIdFile);
            fs = new FileInputStream(f);
            InputStream ins = ((InputStream) fs);
            Scanner scan = new Scanner(ins);
            if (scan.hasNextLine()) {
                Log.d(TAG,"getProvisioningID  from Provisioning_ID File ");
                vendorPartitionValue = scan.nextLine();
            } else {
                Log.d(TAG,"getProvisioningID  NO LINES in  Provisioning_ID File return default -------");
                vendorPartitionValue = "";
            }
            return vendorPartitionValue;
        } catch (FileNotFoundException e) {
            Log.e(TAG,"Failed to Load From provisioning File [FileNotFoundException]");
            return defaultValue;
        } catch (Exception e) {
            Log.e(TAG, "Failed to Load From provisioning File [Exception]");
            return defaultValue;
        } finally {
            Log.d(TAG, "closing Provisioning_ID file");
            if(null != fs) {
                try {
                    fs.close();
                } catch(IOException e) {
                    Log.e(TAG, "closing Provisioning_ID file [IOException]");
                    e.printStackTrace();
                } catch(Exception e) {
                    Log.e(TAG, "closing Provisioning_ID file [Exception]");
                    e.printStackTrace();
                }
            }
        }
    }

    public static final String getPropertyValue(String key, String defaultValue) {
        Properties p;
        try {
            Runtime.getRuntime().exec( "chmod 777 "+ PROPERTIES_FILE );  
            File properties = new File(PROPERTIES_FILE);
            FileInputStream fis = new FileInputStream(properties);
            p = new Properties();
            p.load(fis);
            fis.close();
            Log.d(TAG, "Loading Property : [" + key + "]");
            if (p.containsKey(key)) {
                String valFromFile = (String) p.get(key);
                Log.d(TAG, "From properties File : " + valFromFile);
                return valFromFile.isEmpty() ? defaultValue : valFromFile;
            } else {
                Log.e(TAG, "Failed to Load From Properties File");
                return defaultValue;
            }
        } catch (FileNotFoundException e) {
            Log.e(TAG, "Failed to Load From Properties File [FileNotFoundException]");
            return defaultValue;
        } catch (IOException e) {
            Log.e(TAG, "Failed to Load From Properties File [IOException]");
            return defaultValue;
        } catch (Exception ex) {
            Log.e(TAG,
                    "Exception occurred. Failed to Load From Properties File." + ex.getMessage());
            return defaultValue;
        }
    }
}
