/*
This program is distributed under the terms of the 'MIT license'. The text
of this licence follows...

Copyright (c) 2006 J.D.Medhurst (a.k.a. Tixy)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

/**
@file

@brief Abstract interfrace to a serial port device.
*/

#ifndef DVF99_UPGRADE_H
#define DVF99_UPGRADE_H

#include "serial_port.h"




/**
@defgroup serialport Comms - Serial Port interface
@brief Serial port interface.
@{
*/

/**
@brief Abstract interfrace to a serial port device.
*/
class Dvf99_upgrade
	{

public:
	/**
	Construct a serial port.

	@return The port object, or zero if an error ocurred.
	*/
	Dvf99_upgrade();

	void Send(SerialPort* port);

	void Receive(SerialPort* port);

	int upgrade_DVF99(const char *portName, unsigned baudRate, const char *sourceFile);

	int readDVFPort(SerialPort* port, const char *text, unsigned timeout);

	int bootDVF99Module();

	void doGPIOReset();


	void Error(int error,const char *format, ...);

	
	/**
	Destructor which also performs a Close().
	*/
	~Dvf99_upgrade();


	
		
	};


/** @} */ // End of group

#endif
