/*
This program is distributed under the terms of the 'MIT license'. The text
of this licence follows...

Copyright (c) 2006 J.D.Medhurst (a.k.a. Tixy)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

/**
@file

@brief Implementation of base class for YModemRx and YModemTx.
*/

#include "common.h"
#include "ymodem.h"
#include <stdio.h>



uint16_t YModem::UpdateCRC16(uint16_t crcIn, uint8_t byte)
	{
	uint32_t crc = crcIn;
	uint32_t in = byte|0x100;
	do
		{
		crc <<= 1;
		in <<= 1;
		if(in&0x100)
			++crc;
		if(crc&0x10000)
			crc ^= 0x1021;
		}
	while(!(in&0x10000));
	return crc&0xffffu;
	}


uint16_t YModem::CRC16(const uint8_t* data, size_t size)
	{
	uint32_t crc = 0;
	const uint8_t* dataEnd = data+size;
	while(data<dataEnd)
		crc = UpdateCRC16(crc,*data++);
	crc = UpdateCRC16(crc,0);
	crc = UpdateCRC16(crc,0);
	return crc&0xffffu;
	}


uint8_t YModem::Checksum(const uint8_t* data, size_t size)
	{
	int sum = 0;
	const uint8_t* dataEnd = data+size;
	while(data<dataEnd)
		sum += *data++;
	return sum&0xffu;
	}


int YModem::InChar(unsigned timeout)
	{
	uint8_t c = 0;
	int result = Port.In(&c,1,timeout);

	if(result==1)
		return c;
	if(result==0)
		return ErrorTimeout;
	return result;
	}


void YModem::Cancel()
	{
	const uint8_t CancelString[] = { CAN,CAN,CAN,CAN,CAN };
	Port.Out(CancelString,sizeof(CancelString),1000);
	}
